// TwoButtonFunct.c
// Runs on LM3S811
// Use vectored interrupts to run functions when buttons are pressed.
// One button and function pair is higher priority.
// Daniel Valvano
// June 27, 2011

/* This example accompanies the book
   "Embedded Systems: Real Time Interfacing to the Arm Cortex M3",
   ISBN: 978-1463590154, Jonathan Valvano, copyright (c) 2011

 Copyright 2011 by Jonathan W. Valvano, valvano@mail.utexas.edu
    You may use, edit, run or distribute this file
    as long as the above copyright notice remains
 THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 VALVANO SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 For more information about my classes, my research, and my books, see
 http://users.ece.utexas.edu/~valvano/
 */

// momentary, positive logic button connected to PC6 (trigger on rising edge)
// momentary, positive logic button connected to PD4 (trigger on rising edge)

#define NVIC_EN0_INT2           0x00000004  // Interrupt 2 enable
#define NVIC_EN0_INT3           0x00000008  // Interrupt 3 enable
#define NVIC_EN0_R              (*((volatile unsigned long *)0xE000E100))  // IRQ 0 to 31 Set Enable Register
#define NVIC_PRI0_R             (*((volatile unsigned long *)0xE000E400))  // IRQ 0 to 3 Priority Register
#define GPIO_PORTC_DIR_R        (*((volatile unsigned long *)0x40006400))
#define GPIO_PORTC_IS_R         (*((volatile unsigned long *)0x40006404))
#define GPIO_PORTC_IBE_R        (*((volatile unsigned long *)0x40006408))
#define GPIO_PORTC_IEV_R        (*((volatile unsigned long *)0x4000640C))
#define GPIO_PORTC_IM_R         (*((volatile unsigned long *)0x40006410))
#define GPIO_PORTC_ICR_R        (*((volatile unsigned long *)0x4000641C))
#define GPIO_PORTC_DEN_R        (*((volatile unsigned long *)0x4000651C))
#define GPIO_PORTD_DIR_R        (*((volatile unsigned long *)0x40007400))
#define GPIO_PORTD_IS_R         (*((volatile unsigned long *)0x40007404))
#define GPIO_PORTD_IBE_R        (*((volatile unsigned long *)0x40007408))
#define GPIO_PORTD_IEV_R        (*((volatile unsigned long *)0x4000740C))
#define GPIO_PORTD_IM_R         (*((volatile unsigned long *)0x40007410))
#define GPIO_PORTD_ICR_R        (*((volatile unsigned long *)0x4000741C))
#define GPIO_PORTD_DEN_R        (*((volatile unsigned long *)0x4000751C))
#define SYSCTL_RCGC2_R          (*((volatile unsigned long *)0x400FE108))
#define SYSCTL_RCGC2_GPIOD      0x00000008  // port D Clock Gating Control
#define SYSCTL_RCGC2_GPIOC      0x00000004  // port C Clock Gating Control

void DisableInterrupts(void); // Disable interrupts
void EnableInterrupts(void);  // Enable interrupts
long StartCritical (void);    // previous I bit, disable interrupts
void EndCritical(long sr);    // restore I bit to previous value
void WaitForInterrupt(void);  // low power mode
void Task1(void);             // task1 function eventually defined elsewhere
void Task2(void);             // task2 function eventually defined elsewhere

void PriorityButtons_Init(void){
  volatile unsigned long delay;
  // activate port C and port D
  SYSCTL_RCGC2_R |= SYSCTL_RCGC2_GPIOC+SYSCTL_RCGC2_GPIOD;
  delay = SYSCTL_RCGC2_R;   // allow time to finish activating
  GPIO_PORTC_DIR_R &= ~0x40;// make PC6 in (PC6 button)
  GPIO_PORTC_DEN_R |= 0x40; // enable digital I/O on PC6
  GPIO_PORTC_IS_R &= ~0x40; // PC6 is edge-sensitive (default setting)
  GPIO_PORTC_IBE_R &= ~0x40;// PC6 is not both edges (default setting)
  GPIO_PORTC_IEV_R |= 0x40; // PC6 rising edge event
  GPIO_PORTC_ICR_R = 0x40;  // clear flag6
  GPIO_PORTC_IM_R |= 0x40;  // enable interrupt on PC6
                            // GPIO PortC=priority 1
  NVIC_PRI0_R = (NVIC_PRI0_R&0xFF00FFFF)|0x00200000; // bits 21-23
  GPIO_PORTD_DIR_R &= ~0x10;// make PD4 in (PD4 button)
  GPIO_PORTD_DEN_R |= 0x10; // enable digital I/O on PD4
  GPIO_PORTD_IS_R &= ~0x10; // PD4 is edge-sensitive (default setting)
  GPIO_PORTD_IBE_R &= ~0x10;// PD4 is not both edges (default setting)
  GPIO_PORTD_IEV_R |= 0x10; // PD4 rising edge event
  GPIO_PORTD_ICR_R = 0x10;  // clear flag4
  GPIO_PORTD_IM_R |= 0x10;  // enable interrupt on PD4
                            // GPIO PortD=priority 2
  NVIC_PRI0_R = (NVIC_PRI0_R&0x00FFFFFF)|0x40000000; // bits 29-31
                            // enable interrupts 2 and 3 in NVIC
  NVIC_EN0_R |= (NVIC_EN0_INT2+NVIC_EN0_INT3);
  EnableInterrupts();
}
void GPIOPortC_Handler(void){
  GPIO_PORTC_ICR_R = 0x40;  // acknowledge flag6
  Task1();                  // service SW1
}
void GPIOPortD_Handler(void){
  GPIO_PORTD_ICR_R = 0x10;  // acknowledge flag4
  Task2();                  // service SW2
}

//debug code
volatile unsigned long PriorityGlobal1, PriorityGlobal2, PriorityOOO;
void Task1(void){
  PriorityGlobal1 = PriorityGlobal1 + 1;
}
void Task2(void){
  long sr;
  PriorityGlobal2 = PriorityGlobal2 + 1;
  sr = StartCritical();
  if(PriorityGlobal1 != PriorityGlobal2){
    PriorityOOO = PriorityOOO + 1;// interrupts happened Out Of Order
  }
  EndCritical(sr);
}
int main(void){
  PriorityButtons_Init();   // initialize interrupts and ports
  PriorityGlobal1 = 0;      // number of times Task1 called
  PriorityGlobal2 = 0;      // number of times Task2 called
  PriorityOOO = 0;          // number of Out Of Order interrupts
                            // where Task2() happened before Task1()
  while(1){
    WaitForInterrupt();
  }
}
